#! /bin/bash
# Set up Axyl
# This script is intended for use when you have grabbed a CVS copy
# of the Axyl system, and need to set up the config file(s) as if
# it had been installed from its Debian package - a sort of
# developer's hack if you will.
#
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
set -e

echo ""
echo "AXYL SETUP"
echo ""
echo "Note: this is generally used by Developers to install a copy"
echo "of Axyl which has been obtained from CVS or a tarball. Make sure"
echo "you know what you're doing - possibly you want to run the"
echo "create-axyl-website.sh to create a new website instead?"
echo ""
echo -n "Continue? [N] :"
read ANS
if [ "$ANS" != "y" -a "$ANS" != "Y" ]
then
  echo "Ok, aborting.."
  exit 0
fi

# Defaults..
AXUSER=
AXYL_HOME=
AXYL_DATA=
AXYL_LOGS=
LUCENE_PORT=
AXYL_VERSION="unknown"

CONFDIR=/etc/axyl
CONF=${CONFDIR}/axyl.conf
if [ -f $CONF ]
then
  . $CONF
  # But if they are invalid dirs, then nullify them again..
  [ ! -d $AXYL_HOME ] && AXYL_HOME=
  [ ! -d $AXYL_DATA ] && AXYL_DATA=
  [ ! -d $AXYL_LOGS ] && AXYL_LOGS=
fi

VERSIONFILE=${CONFDIR}/axyl.version
if [ -f $VERSIONFILE ]
then
  AXYL_VERSION=`cat $VERSIONFILE`
  if [ "$AXYL_VERSION" = "" ]
  then
    AXYL_VERSION="unknown"
  fi
fi

if [ "$AXYL_HOME" != "" ]
then
  echo "Axyl seems to be already installed."
  echo "The config file ($CONF) says it is in $AXYL_HOME"
  echo ""
  echo -n "Do you want to re-write the configuration file? [N] :"
  read ANS
  if [ "$ANS" != "y" -a "$ANS" != "Y" ]
  then
    echo "Ok, aborting.."
    exit 0
  fi

  # Nullify old config file. Definitions come from
  # the include of axyl-common.sh
  if [ -d $CONFDIR ]
  then
    rm -f $CONF
  fi
  AXYL_HOME=
  AXYL_DATA=
  AXYL_LOGS=
fi

echo ""
echo "This script will initialise the Axyl config file to point to"
echo "the current Axyl location, and also optionally install the"
echo "Lucene Server for you."

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# AXYL INSTALLATION LOCATION
HERE=`pwd`
cd ..
AXYL_HOME=`pwd`
cd $HERE
echo ""
echo "Axyl Location"
echo "Enter the full path of the Axyl distribution.."
echo -n "[$AXYL_HOME] :"
read ANS
[ "$ANS" != "" ] && AXYL_HOME=$ANS

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Get common funcs etc..
if [ -e ./axyl-common-funcs.sh ]
then
  . ./axyl-common-funcs.sh
else
  echo "ERROR: axyl-common-funcs.sh not found or not executable. You must"
  echo "execute this from within the Axyl 'install' directory, or else"
  echo "fix file permissions."
  exit 1
fi

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Detect Apache and Php settings
# These two detection scripts set up environment vars for use by
# other scripts.
if [ "$AXYL_HOME" != "" ] ; then

	# Detect the APACHE configuration settings. Sets up the following vars:
	# APACHE_NAME     # Name of the apache instance eg. 'apache2'
	# APACHE_CONFDIR  # Configuration files live here
	# APACHE_CONFSYS  # Type of config system: 'ap2', 'confd' or 'httpd'
	# APACHE_CONF     # Path to main apache configuration file
	# APACHE_USER     # User apache runs as eg. 'www-data'
	# APACHE_GROUP    # Group apache runs under eg. 'www-data'
	if [ -f ${AXYL_HOME}/install/detect-apache.sh ] ; then
		. ${AXYL_HOME}/install/detect-apache.sh
	fi

	# Detect the PHP configuration settings. Sets up the following vars:
	# PHP             # The 'preferred' version we found
	# PHP_BIN         # The path to the php binary itself
	# PHP_INI_APACHE  # Path to Apache ini file
	# PHP_INI_CLI     # Path to CLI ini file
	# PHPDET          # Flag: =1 if we found at least one Php
	# PHP_APACHE_MOD  # Flag: =1 if apache module is installed
	if [ -f ${AXYL_HOME}/install/detect-php.sh ] ; then
  		. ${AXYL_HOME}/install/detect-php.sh
	fi
fi

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# USERS AND GROUPS

# Set up the required users and groups
AXUSER=axyl
echo -n "Enter the Axyl administrator user [$AXUSER] :"
read ANS
[ "$ANS" != "" ] && AXUSER=$ANS

# The Axyl user..
mkuser $AXUSER system $AXUSER any $AXYL_HOME default "Axyl Administrator"

# Webserver user and group..
WWWUSER=www-data
WWWGROUP=www-data
WWWADMIN=wwwadmin
if [ -r ${APACHE_CONFDIR}/httpd.conf} ] ; then
  G=`grep "^Group " ${APACHE_CONFDIR}/httpd.conf | cut -d" " -f2`
  U=`grep "^User " ${APACHE_CONFDIR}/httpd.conf | cut -d" " -f2`
  [ "$G" != "" ] && WWWGROUP=$G
  [ "$U" != "" ] && WWWUSER=$U
fi
mkuser $WWWUSER system $WWWGROUP any /var/www default "WWW User"

# Web hosting administration group..
mkgroup $WWWADMIN any

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# AXYL VAR DATA AREA
# Tha main usage of this area is by the Luceneserver which comes
# with Axyl, and stores Lucene configs, and indexes here. The
# Lucene indexes can get big, so symlink as necessary to manage
# that aspect of the operation.
AXYL_DATA=/var/lib/axyl
echo ""
echo "Setting up the Axyl data area. This is where Lucene will store"
echo "varying (growing) data such as Lucene indexes, so make sure you"
echo "pick a location with that in mind."
echo ""
echo "Enter the full path of the Axyl data area.."
echo -n "[$AXYL_DATA] :"
read ANS
[ "$ANS" != "" ] && AXYL_DATA=$ANS

# Create directory and set perms..
mkthisdir $AXYL_DATA
chown -R ${AXUSER}:${AXUSER} ${AXYL_DATA}
find ${AXYL_DATA} -type f -exec chmod 0664 {} \;
find ${AXYL_DATA} -type d -exec chmod 6755 {} \;

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# AXYL LOGGING AREA
if [ -d /var/log ]
then
  AXYL_LOGS=/var/log/axyl
else
  AXYL_LOGS=${AXYL_DATA}/log
fi

# Create directory and set perms..
mkthisdir $AXYL_LOGS
chown -R ${AXUSER}:${AXUSER} ${AXYL_LOGS}
find ${AXYL_LOGS} -type f -exec chmod 0664 {} \;
find ${AXYL_LOGS} -type d -exec chmod 0755 {} \;

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# RE-WRITE CONFIGURATION
# Write out the config file for Axyl Lucene installer to use..
[ -z $LUCENE_PORT ] && LUCENE_PORT="null"
rewrite_axylconf $AXUSER $AXYL_HOME $AXYL_DATA $AXYL_LOGS $LUCENE_PORT

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# LUCENE (RE-)INSTALLATION
# This will (re-)install the Lucene Server, even if it is already
# installed. That way we ensure paths are re-written. But, we only
# do this if Axyl Lucene is installed..
if [ -e ${AXYL_HOME}/lucene ]
then
  LUCENE_INSTALLER=${AXYL_HOME}/lucene/install/install-lucene.sh
  if [ -x $LUCENE_INSTALLER ]
  then
    echo ""
    echo "(re-)installing Axyl Lucene.."
    LUCENE_OVERWRITE=y
    LUCENE_DEBCONF=n
    sh $LUCENE_INSTALLER $LUCENE_PORT $LUCENE_OVERWRITE $LUCENE_DEBCONF
  fi
fi

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# FINISH UP
# Confirm directly from the configuration file itself..
if [ -f $CONF ]
then
  # Read everything back..
  . $CONF

  # Always re-write version file if we can establish version..
  # (nb: some vars below are defined in axyl-common.sh)
  CHGLOG=../debian/changelog
  if [ -f $CHGLOG ]
  then
    echo ""
    echo "re-writing the Axyl version file.."
    NEW_AXYL_VERSION=`head -n 1 $CHGLOG | cut -f2 -d\( | cut -f1 -d')' | cut -f1 -d-`
    echo $NEW_AXYL_VERSION >$VERSIONFILE
    if [ "$AXYL_VERSION" != "$NEW_AXYL_VERSION" ] ; then
      if [ "$AXYL_VERSION" = "" ] ; then
        AXYL_VERSION=unset
      fi
      echo "Note: version is now $NEW_AXYL_VERSION (was $AXYL_VERSION)"
      AXYL_VERSION=$NEW_AXYL_VERSION
    fi
  else
    echo "ERROR: could not locate Debian 'changelog' file. Are you running"
    echo "this script from AXYL_HOME/install/ ?"
  fi

  # Assert proper ownership & perms..
  chown -R ${AXUSER}:${AXUSER} $CONFDIR
  chmod -R 0644 $CONFDIR
  find $CONFDIR -type d -exec chmod 0775 {} \;

  # Read the current version file..
  AXYL_VERSION=`cat $VERSIONFILE`
  if [ "$AXYL_VERSION" = "" ]
  then
    AXYL_VERSION="unknown"
  fi

  echo ""
  echo "Configuration summary"
  echo " Axyl version: $AXYL_VERSION"
  echo " Axyl administrator: $AXUSER"
  echo " Axyl installation home: $AXYL_HOME"
  echo " Axyl data area: $AXYL_DATA"
  echo " Axyl logging area: $AXYL_LOGS"
  echo " Lucene port: $LUCENE_PORT"
  echo ""
  echo "To create website(s) from now on, you need to execute the script"
  echo "$AXYL_HOME/install/create-axyl-website.sh"

else
  echo "Axyl was NOT set up successfully."
fi

# END